function [ESlarvae]=broadcastspawning(REEF,REEFpob,Fb,type)
%
%   function [ESlarvae]=broadcastspawning(REEF,REEFpob,Fb,type)
%
%   Create new larvae by external sexual reproduction. Cross-over operation
%   
%   Input - REEF: coral reef, REEFpob: reef population, Fb: fraction of
%   broadcast spawners with respect to the overall amount of existing
%   corals, type: type of crossover depending on the type. type can be set to one of these options ('cont', 'disc',
%   'bin')
%   Output - ESlarvae: created larvae


%get  number of spawners, forzed to be even (pairs of corals to create one
%larva)
nspawners=round(Fb*sum(sum(REEF)));
if mod(nspawners,2)~=0
    nspawners=nspawners-1;
end

%get spawners and divide them in two groups
p=find(REEF~=0);
a=randperm(length(p));
spawners=p(a(1:nspawners));
spawners1=REEFpob(spawners(1:nspawners/2),:);
spawners2=REEFpob(spawners(nspawners/2+1:end),:);


%get crossover mask for some of the methods below (one point crossover)
[a,b]=size(spawners1);
ind=b*[0:a-1]+ceil(b*rand(1,a));
mask=zeros(b,a);
mask(ind)=1;
mask=cumsum(mask);
mask=mask';

if strcmp(type,'cont')
    cruce='SBX'; %set to the desired one
    if strcmp(cruce,'SBX')
        %Simulated Binary Crossover
        eta=2; %para la SBX
        U=rand(size(spawners1));
        B=(2*U).^(1/(eta+1));
        
        ESlarvae1=spawners1.*(1-mask).*(1-B)/2+spawners2.*mask.*(1+B)/2;
        ESlarvae2=spawners1.*(1-mask).*(1+B)/2+spawners2.*mask.*(1-B)/2;
    elseif strcmp(cruce,'aritmetico')
        alpha=0.5;
        ESlarvae1=alpha*spawners1+(1-alpha)*spawners2;
        ESlarvae2=alpha*spawners2+(1-alpha)*spawners1;
    elseif strcmp(cruce,'geometrico')
        ESlarvae1=sqrt(spawners1.*spawners2);
        ESlarvae2=[];
    elseif strcmp(cruce,'uniforme')
        mask=round(rand(size(spawners1)));
        ESlarvae1=mask.*spawners1+(1-mask).*spawners2;
        ESlarvae2=mask.*spawners2+(1-mask).*spawners1;
    end
elseif strcmp(type,'disc') || strcmp(type,'bin')
    ESlarvae1=spawners1.*(1-mask)+spawners2.*mask;
    ESlarvae2=spawners2.*(1-mask)+spawners1.*mask;
end

ESlarvae=[ESlarvae1;ESlarvae2];