function [ISlarvae]=brooding(REEF,REEFpob,Fb,type)
%
%   function [ISlarvae]=brooding(REEF,REEFpob,Fb);
%
%   Create new larvae by internal sexual reproduction.
%   
%   Input - REEF: coral reef, REEFpob: reef population, Fb: fraction of
%   broadcast spawners with respect to the overall amount of existing
%   corals, type: type of crossover depending on the type. type can be set to one of these options ('cont', 'disc',
%   'bin')
%   Output - ISlarvae: created larvae

%get the brooders
nbrooders=round((1-Fb)*sum(sum(REEF)));
p=find(REEF~=0);
a=randperm(length(p));
brooders=p(a(1:nbrooders));
brooders=REEFpob(brooders,:);

ISlarvae=zeros(size(brooders));

if strcmp(type,'bin')
    ISlarvae=mod(brooders+round(rand(size(brooders))),2);
elseif strcmp(type,'disc')
    %xij=xij·(1+0.1·U(-1,1))
    ISlarvae=round(brooders.*(1+0.25*(2*rand(size(brooders))-1)));
elseif strcmp(type,'cont')
    eta=0.2;
    M=max(brooders);
    m=min(brooders);
    s=abs(M-m);
    S=repmat(s,size(brooders,1),1);
    r=rand(size(brooders));
    D=zeros(size(r));
    D(r<0.5)=(2*r(r<0.5)).^(1/(eta+1))-1;
    D(r>=0.5)=1-(2*(1-r(r>=0.5))).^(1/(eta+1));
    ISlarvae=brooders+S.*D;
end
