function [REEF,REEFpob,REEFfitness]=larvaesetting(REEF,REEFpob,REEFfitness,larvae,larvaefitness,k0,opt)
%
%   function [REEF,REEFpob]=larvaesetting(REEF,REEFpob,ESlarvae,ISlarvae)
%
%   Settle the best larvae in the reef, eliminating those which are not
%   good enough
%   
%   Input - REEF: coral reef, REEFpob: reef population, REEFfitness: reef
%   fitness, larvae: larvae population, larvaefitness: larvae fitness, k0:
%   number of oportunities for each larva to settle in the reef, opt: type
%   of optimization ('min' or 'max')
%   Output - REEF: new coral reef, REEFpob: new reef population, REEFfitness: new reef
%   fitness


[Nlarvae,~]=size(larvae);
[P,~]=size(REEFpob);
a=randperm(Nlarvae);
larvae=larvae(a,:);
larvaefitness=larvaefitness(a);

%Each larva is assigned a place in the reef to settle
nreef=randperm(P);
nreef=nreef(1:Nlarvae);

%larvae occupies empty places
free=intersect(nreef,find(REEF==0));
REEF(free)=1;
REEFpob(free,:)=larvae(1:length(free),:);
REEFfitness(free)=larvaefitness(1:length(free));
larvae(1:length(free),:)=[];
larvaefitness(1:length(free))=[];

%in the occupied places a fight for the space is produced
nreef=randperm(P);
ocup=intersect(nreef,find(REEF==1));
[Nlarvae,~]=size(larvae);
while Nlarvae~=0
    k=k0;
    ok=0;
    while k~=0 && ok==0
        ind=ceil(length(ocup)*rand());
        if strcmp(opt,'max')
            %check if the larva is better than the installed coral
            if larvaefitness(1)>REEFfitness(ocup(ind))
                %settle the larva
                REEFpob(ocup(ind),:)=larvae(1,:);
                REEFfitness(ocup(ind))=larvaefitness(1);
                %eliminate the larva from the larvae list
                larvaefitness(1)=[];
                larvae(1,:)=[];
                %eliminate the place from the occupied ones
                ocup(ind)=[];
                ok=1;
            end
        else
            %check if the larva is better than the installed coral
            if larvaefitness(1)<REEFfitness(ocup(ind))
                %settle the larva
                REEFpob(ocup(ind),:)=larvae(1,:);
                REEFfitness(ocup(ind))=larvaefitness(1);
                %eliminate the larva from the larvae list
                larvaefitness(1)=[];
                larvae(1,:)=[];
                %eliminate the place from the occupied ones
                ocup(ind)=[];
                ok=1;
            end
        end   
        k=k-1;
    end
    if ok==0 %eliminate the larva
        larvae(1,:)=[];
        larvaefitness(1)=[];
    end
    [Nlarvae,~]=size(larvae);
end