%%%
%%% CRO max ones

home;
clear;
close all;

%Parameters initialization
Ngen=500;                  % Number of generations
N1=10;                      % Reef size 1
N2=10;                      % Reef size 2
Fb=0.9;                     % Broadcast prob.
Fa=0.01;                    % Asexual reproduction prob.
Fd=Fa;                      % Fraction of the corals to be eliminated in the depredation operator.
r0=0.6;                     % Free/total initial proportion
k=3;                        % Number of opportunities for a new coral to settle in the reef
Pd=0.1;                     % Depredation prob.
opt='max';                  % flag: 'max' for maximizing and 'min' for minimizing


L=100;

Meanfitness=zeros(1,Ngen);
Bestfitness=Meanfitness;

%Reef initialization
[REEF,REEFpob]=reefinitialization(N1,N2,r0,L);
[REEFfitness]=fitness(REEFpob);

for n=1:Ngen
    %Broadcast spawning (External Sexual Reproduction)
    [ESlarvae]=broadcastspawning(REEF,REEFpob,Fb,'bin');
    %Brooding (Internal Sexual Reproduction)
    [ISlarvae]=brooding(REEF,REEFpob,Fb,'bin');
    %Larvae correction 
    [ESlarvae]=larvaecorrection(ESlarvae);
    [ISlarvae]=larvaecorrection(ISlarvae);
    %larvae fitness
    [ESfitness]=fitness(ESlarvae);
    [ISfitness]=fitness(ISlarvae);
    %Larvae setting
    [REEF,REEFpob,REEFfitness]=larvaesetting(REEF,REEFpob,REEFfitness,[ESlarvae;ISlarvae],[ESfitness;ISfitness],k,opt);
    %Asexual reproduction
    [Alarvae,Afitness]=budding(REEF,REEFpob,REEFfitness,Fa,opt);
    [REEF,REEFpob,REEFfitness]=larvaesetting(REEF,REEFpob,REEFfitness,Alarvae,Afitness,k,opt);
    if n~=Ngen
        %Extreme depredation operator for avoiding elitism
        [REEF,REEFpob,REEFfitness]=extremedepredation(REEF,REEFpob,REEFfitness,round(0.2*N1*N2));
        %[REEF,REEFpob,REEFfitness]=depredation(REEF,REEFpob,REEFfitness,Fd,Pd,'max');
    end
    home;
    disp([num2str(n/Ngen*100) '% completado']);
    
    if strcmp(opt,'max')
        Bestfitness(n)=max(REEFfitness(REEFfitness~=0));
    else
        Bestfitness(n)=min(REEFfitness(REEFfitness~=0));
    end
    Meanfitness(n)=mean(REEFfitness(REEFfitness~=0));
end

if sum(sum(REEF))~=(N1*N2)
    REEFfitness(REEF==0)=[];
    REEFpob(REEF==0,:)=[];
end

if strcmp(opt,'max')
    Bestfitness(n)=max(REEFfitness(REEFfitness~=0));
    [m,best]=max(REEFfitness);
else
    Bestfitness(n)=min(REEFfitness(REEFfitness~=0));
    [m,best]=min(REEFfitness);
end

optind=REEFpob(best,:);

disp(num2str(m))

figure;
plot(1:Ngen,Bestfitness,1:Ngen,Meanfitness);
ylim([0 100]);
legend('Best fitness', 'Mean fitness','location','best');